/*
 * Copyright (c) 2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef VELODYNE_BUFFER_H
#define VELODYNE_BUFFER_H

/* System headers */
#include <pthread.h>
#include <stdbool.h>
#include <sys/time.h> /* Necessary for velodyneLib.h */
#include <sys/types.h> /* Necessary for velodyneLib.h */

/* External headers */
#include <velodyne/velodyneLib.h>

/* External Genom headers */
#include <pomStruct.h>

/* Internal headers */
#include "velodyne3DImage.h"
#include "velodyneStruct.h"
#include "velodyneTarget.h"

/* Structures */
typedef struct velodynePacket {
  POM_SENSOR_POS  position;
  VELODYNE_PACKET packet;
} velodynePacket;

typedef struct velodyneBuffer {
  int             read;
  int             write;
  int             nbPackets;
  bool            dataOverwriting;
  velodynePacket* packets;
} velodyneBuffer;

typedef struct velodyneReadTask {
  pthread_t       thr;
  pthread_mutex_t mtx;
  velodyneBuffer  buffer;
  bool            needOneShot;
  pthread_cond_t  cond;
} velodyneReadTask;

/* Errors */
#define VELODYNE_READER_OK 0
#define VELODYNE_READER_ERR_CANNOT_OPEN_POS_FILE 1
#define VELODYNE_READER_ERR_BAD_POS_FILE_FORMAT 2
#define VELODYNE_READER_ERR_END_POS_FILE 3
#define VELODYNE_READER_ERR_CANNOT_READ_POS_FILE 4

/* Functions */
void velodyneCreateBuffer(int numPackets);

void velodyneDestroyBuffer();

void* velodyneReadLive();

/*
 * Errors:
 * VELODYNE_READER_ERR_CANNOT_OPEN_POS_FILE
 * VELODYNE_READER_ERR_BAD_POS_FILE_FORMAT
 */
int velodynePosFileOpen(const char* fileName, FILE** fileId);

/*
 * Errors:
 * VELODYNE_READER_ERR_END_POS_FILE
 * VELODYNE_READER_ERR_CANNOT_READ_POS_FILE
 */
int velodynePosFileRead(FILE* fileId, double* timespec, POM_EULER* pos);

void* velodyneReadPlayback(void* positionFileId);

ACTIVITY_EVENT velodyneGrabOneShot(velodyneBuffer*,
                                   velodyneOneShotParams*,
                                   velodyne3DImage*,
                                   velodyne3DImageParams*,
                                   velodyneTargetDetectionParams*,
                                   velodyneTargetParams*,
                                   velodyneTarget*);

#endif
